package lightcontainer.storage;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;

public class Repository {

    private final String filePath;
    private final Adapter adapter;
    private final AppData appData;

    /**
     * @param filePath The path to the configuration file
     * @param adapter  The adapter that converts objects to string and vice-versa
     */
    public Repository(String filePath, AppData appData, Adapter adapter) {
        this.filePath = filePath;
        this.appData = appData;
        this.adapter = adapter;
    }

    /**
     * Saves configuration file
     */
    public void save() {
        if (filePath != null) {
            String jsonAppData = adapter.toString();
            try (BufferedWriter bufferedWriter = Files.newBufferedWriter(Paths.get(filePath).toAbsolutePath(), StandardCharsets.UTF_8, StandardOpenOption.CREATE, StandardOpenOption.TRUNCATE_EXISTING)) {
                bufferedWriter.write(jsonAppData);
                bufferedWriter.flush();
            } catch (IOException e) {
                System.out.println("Error while saving configuration file !");
            }
        }
    }

    public boolean addUser(String login, String password, String key) {
        if (appData.addUser(login, password, key)) {
            save();
            return true;
        }
        return false;
    }

    public boolean addFileFor(File file, String userName) {
        if (appData.addFileFor(file, userName)) {
            save();
            return true;
        }
        return false;
    }

    public boolean deleteFileOf(String fileName, String userName) {
        if (appData.deleteFileOf(fileName, userName)) {
            save();
            return true;
        }
        return false;
    }

    public boolean addStorage(String userName, File file, String storage) {
        if (appData.addStorage(userName, file, storage)) {
            save();
            return true;
        }
        return false;
    }


    /**
     * Loads configuration file
     *
     * @return AppData
     */
    public AppData load() {
        String jsonString = readFile(filePath);
        return adapter.fromString(jsonString);
    }

    private String readFile(String filePath) {
        StringBuilder builder = new StringBuilder();
        try (BufferedReader reader = Files.newBufferedReader(Paths.get(filePath).toAbsolutePath(), StandardCharsets.UTF_8)) {
            while (reader.ready()) {
                builder.append(reader.readLine());
            }
        } catch (IOException e) {
            System.out.println("Error while reading configuration file");
            builder.setLength(0);
        }
        return builder.toString();
    }
}
